package io.sfbx.appconsent_tv

import androidx.annotation.NonNull
import com.sfbx.appconsent.AppConsent
import com.sfbx.appconsent.tv.AppConsentSDK
import com.sfbx.appconsent.tv.model.ACConfiguration
import com.sfbx.appconsent.tv.model.ACConsentableType
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

class AppconsentTvPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var channel: MethodChannel
    private var ackit: AppConsent? = null

    override fun onDetachedFromActivity() {
        ackit = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {

    }

    override fun onDetachedFromActivityForConfigChanges() {
        ackit = null
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, channelName)
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        ackit = null
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "setup" -> setup(call, result)
            "presentNotice" -> presentNotice(call, result)
            "clearConsent" -> clearConsent(result)
            "checkForUpdate" -> checkForUpdate(result)
            "isSubjectToGDPR" -> isSubjectToGDPR(result)
            "consentGiven" -> consentGiven(result)
            "geolocationConsentGiven" -> geolocationConsentGiven(result)
            "consentableAllowed" -> consentableAllowed(call, result)
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun onReady(appConsent: AppConsent, result: Result) {
        this.ackit = appConsent
        result.success(null)
    }

    private fun setup(call: MethodCall, result: Result) {
        val appKey: String = call.argument("appKey") ?: ""
        val forceApplyGDPR: Boolean = call.argument("forceApplyGDPR") ?: false

        AppConsentSDK.initialize(
            appKey = appKey,
            configuration = ACConfiguration.Builder()
                .setForceApplyGDPR(forceApplyGDPR)
                .build()
        ) { (this::onReady)(it, result) }
    }

    private fun presentNotice(call: MethodCall, result: Result) {
        val force: Boolean = call.argument("force") ?: false
        ackit?.tryToDisplayNotice(force)
        result.success(null)
    }

    private fun clearConsent(result: Result) {
        ackit?.clearConsent()
        result.success(null)
    }

    private fun checkForUpdate(result: Result) {
        ackit?.checkForUpdate(onResult = {
            result.success(it)
        },
            error = {
                result.success(false)
            })
    }

    private fun isSubjectToGDPR(result: Result) {
        val isSubjectToGDPR = ackit?.isSubjectToGDPR()
        result.success(isSubjectToGDPR)
    }

    private fun consentGiven(result: Result) {
        val consentGiven = ackit?.consentGiven()
        result.success(consentGiven)
    }

    private fun geolocationConsentGiven(result: Result) {
        val geolocationConsentGiven = ackit?.geolocationConsentGiven()
        result.success(geolocationConsentGiven)
    }

    private fun consentableAllowed(call: MethodCall, result: Result) {
        val iabId: Int = call.argument("iabId") ?: 0
        val type: Int = call.argument("consentableType") ?: 0
        val consentableType: ACConsentableType = when (type) {
            0 -> ACConsentableType.PURPOSE
            1 -> ACConsentableType.FEATURE
            2 -> ACConsentableType.SPECIAL_FEATURE
            3 -> ACConsentableType.SPECIAL_PURPOSE
            else -> ACConsentableType.UNKNOWN
        }
        val isAllowed = ackit?.consentableAllowed(iabId, consentableType)
        result.success(isAllowed)
    }

    companion object {
        private val channelName = "appconsent_tv"
    }
}
